#define RICO_APPROVES 0
//******************************************************************************
//
//	File:		SerialPort.cpp
//
//	Description:	BSerialPort class.
//			Implementation for a Serial Port class.
//	
//	Written by:	Erich Ringewald
//
//	Copyright 1995, Be Incorporated, All Rights Reserved.
//
//
//******************************************************************************

#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>
#include <unistd.h>
#include <termios.h>
#include <errno.h>

#include "serial.h"

status_t Ioctl(int fd, int opcode, void *buf);

//------------------------------------------------------------

status_t CreateSerialPort(serial_port **aPort)
{
	*aPort = 0;
	serial_port *newport = (serial_port *)malloc(sizeof(serial_port));
	if (!newport)
		return B_ERROR;
			
	newport->ffd = -1;
	newport->fBaudRate = B_9600_BPS;				// set default line character
	newport->fDataBits = B_DATA_BITS_8;
	newport->fStopBits = B_STOP_BITS_1;
	newport->fParityMode = B_NO_PARITY;
	newport->fBlocking = TRUE;						// Default is blcoking reads
	newport->fTimeout = B_INFINITE_TIMEOUT;			// and to block forever
	newport->fFlow = B_HARDWARE_CONTROL;				// with HW flow

	*aPort = newport;
}

void
DestroySerialPort(serial_port *aPort)
{
	if (aPort->ffd > -1)
		close(aPort->ffd);
	free(aPort);
}

// Actually open the port.  Since this might fail, we have to do this
// outside of the constructor.
// valid port names are "serial1", "serial2", "serial3", and "serial4"

status_t
SP_Open(serial_port &aPort, const char *portName)
{
	char  buf[64];

	if (*portName == '/') {
		sprintf(buf, "%s", portName);
	} else {
		//sprintf(buf, "/dev/ports/%s", portName);		// prepend "/dev/"
		sprintf(buf, "/dev/%s", portName);		// prepend "/dev/"
	}

	if (aPort.ffd >= 0)								// it's already open
		close(aPort.ffd);								// close to reopen

	aPort.ffd = open(buf, O_RDWR);					// open the port
	if (aPort.ffd >= 0)								// if it worked
		SP_DriverControl(aPort);						// make it so

	return(aPort.ffd);
}

void
SP_Close(serial_port &aPort)
{
	close(aPort.ffd);
	aPort.ffd = -1;
}

ssize_t
SP_Read(serial_port &aPort, void *buf, size_t count)
{
	return (read( aPort.ffd, buf, count));
}

ssize_t
SP_Write(serial_port &aPort, const void *buf, size_t count)
{
	return write(aPort.ffd, buf, count);
}

void
SP_SetBlocking(serial_port &aPort, bool BlockingFlag)
{
	aPort.fBlocking = BlockingFlag;
	SP_DriverControl(aPort);
}

status_t
SP_SetTimeout(serial_port &aPort, bigtime_t Timeout)
{

	//if ((Timeout == B_INFINITE_TIMEOUT) || (Timeout/100000 < 256 && Timeout >= 0)) 
	//if ((Timeout > 1000000000) || (Timeout/100000 < 256 && Timeout >= 0)) 
	{
		aPort.fTimeout = Timeout;
		SP_DriverControl(aPort);
	
		return (B_OK);
	}

	return (B_BAD_VALUE);
}

// Set the BaudRate.

status_t
SP_SetDataRate(serial_port &aPort, data_rate bps)
{
	aPort.fBaudRate = bps;
	return (SP_DriverControl(aPort));
}

data_rate
SP_DataRate(serial_port &aPort)
{
	return(aPort.fBaudRate);
}

void
SP_SetDataBits(serial_port &aPort, data_bits numBits)
{
	aPort.fDataBits = numBits;
	SP_DriverControl(aPort);
}

data_bits
SP_DataBits(serial_port &aPort)
{
	return(aPort.fDataBits);
}

void
SP_SetStopBits(serial_port &aPort, stop_bits numBits)
{
	aPort.fStopBits = numBits;
	SP_DriverControl(aPort);
}

stop_bits
SP_StopBits(serial_port &aPort)
{
	return(aPort.fStopBits);
}

void
SP_SetParityMode(serial_port &aPort, parity_mode pm)
{
	aPort.fParityMode = pm;
	SP_DriverControl(aPort);
}

parity_mode
SP_ParityMode(serial_port &aPort)
{
	return(aPort.fParityMode);
}

void
SP_ClearInput(serial_port &aPort)
{
	ioctl(aPort.ffd, TCFLSH, 0);
	return;
}

void
SP_ClearOutput(serial_port &aPort)
{
	ioctl(aPort.ffd, TCFLSH, 1);
	return;
}

void
SP_SetFlowControl(serial_port &aPort, uint32 mask)
{
	aPort.fFlow = mask & (B_HARDWARE_CONTROL | B_SOFTWARE_CONTROL);
	SP_DriverControl(aPort);
}

uint32
SP_FlowControl(serial_port &aPort)
{
	return aPort.fFlow;
}

status_t
Ioctl(int fd, int opcode, void *buf)
{
	status_t res;

	res = ioctl(fd, opcode, buf);
	if (res < 0) {
		return (errno);
	}
	return (res);
}

//
// Send a control message to the driver to ensure that the current
// settings are set.

int
SP_DriverControl(serial_port &aPort)
{
struct termio	tctl;
int				error;

	if (aPort.ffd < 0)						// cant do this without a driver
		return (B_NO_INIT);

	error = ioctl(aPort.ffd, TCGETA, &tctl);			// get the current settings
	if (error < 0)
		return errno;

	// turn off all of the bits which we will turn on again, if necessary

	tctl.c_iflag &= ~(IXON | IXOFF | IXANY | INPCK);
	tctl.c_cflag &= ~(CSIZE | CBAUD | CSTOPB | PARENB | PARODD | 
														CTSFLOW | RTSFLOW);
	tctl.c_lflag &= ~(ICANON | ECHO | ECHONL | ISIG);
#if RICO_APPROVES
	tctl.c_cc[VMIN] = fBlocking;
	tctl.c_cc[VTIME] = fTimeout / 100000;
#else 
	if (aPort.fBlocking) {
		if (aPort.fTimeout == B_INFINITE_TIMEOUT) {
			tctl.c_cc[VMIN] = 1;
			tctl.c_cc[VTIME] = 0;
		}
		else if (aPort.fTimeout == 0)
			tctl.c_cc[VMIN] = 0;
		else {
			int i = aPort.fTimeout / 100000;
			if (i == 0)
				i = 1;
			tctl.c_cc[VMIN] = 1;
			tctl.c_cc[VTIME] = i;
		}
	}
	else {
		tctl.c_cc[VMIN] = 0;
	}
#endif

	if (aPort.fFlow & B_SOFTWARE_CONTROL)
		tctl.c_iflag |= (IXON | IXOFF);

	if (aPort.fFlow & B_HARDWARE_CONTROL)
		tctl.c_cflag |= (CTSFLOW | RTSFLOW);

	if (aPort.fDataBits == B_DATA_BITS_8)
		tctl.c_cflag |= CS8;	
	if (aPort.fStopBits == B_STOP_BITS_2)
		tctl.c_cflag |= CSTOPB;	
	if (aPort.fParityMode != B_NO_PARITY) {
		tctl.c_cflag |= PARENB;
		if (aPort.fParityMode == B_ODD_PARITY)
			tctl.c_cflag |= PARODD;
	}
	tctl.c_cflag |= (aPort.fBaudRate & CBAUD);
	return Ioctl(aPort.ffd, TCSETA, &tctl);
}

status_t
SP_SetDTR(serial_port &aPort, bool assert)
{
	return Ioctl(aPort.ffd, TCSETDTR, &assert);
}

status_t
SP_SetRTS(serial_port &aPort, bool assert)
{
	return Ioctl(aPort.ffd, TCSETRTS, &assert);
}

bool
SP_IsCTS(serial_port &aPort)
{
unsigned int	thebits;

	thebits = ioctl(aPort.ffd, TCGETBITS, 0);
	if (thebits & TCGB_CTS)
		return true;
	else
		return false;
}

bool
SP_IsDSR(serial_port &aPort)
{
unsigned int	thebits;

	thebits = ioctl(aPort.ffd, TCGETBITS, 0);
	if (thebits & TCGB_DSR)
		return true;
	else
		return false;
}

bool
SP_IsRI(serial_port &aPort)
{
unsigned int	thebits;

	thebits = ioctl(aPort.ffd, TCGETBITS, 0);
	if (thebits & TCGB_RI)
		return true;
	else
		return false;
}

bool
SP_IsDCD(serial_port &aPort)
{
unsigned int	thebits;

	thebits = ioctl(aPort.ffd, TCGETBITS, 0);
	if (thebits & TCGB_DCD)
		return TRUE;
	else
		return FALSE;
}

ssize_t
SP_WaitForInput(serial_port &aPort)
{
	int len;
	int status;

	status = ioctl(aPort.ffd, TCWAITEVENT, &len, sizeof(len));
	if (status < B_NO_ERROR) 
	{
		return (errno);
	}
	return (len);
}

