#include "tablet.h"

#include <stdio.h>
#include <string.h>
#include <unistd.h>

// Utility Commands
void		GetLine(serial_port &port, char *buff, const long buffLen);
status_t	GetBytes(serial_port &port, uint8 *buff, const uint32 buffLen);
status_t	SetTransmissionInterval(const int16);

int	SendCommand(wacom_tablet &tablet, const char *cmd);

static status_t convertpositiontostruct(const uint8 *data, tablet_position &);
static status_t convertinfotostruct(const uint32 *data, tablet_info &);

const uint8 kSyncBit		= 7;
const uint8 kProximityBit 	= 6;
const uint8 kPointerBit		= 5;
const uint8 kButtonBit		= 3;
const uint8 kPressureSignBit	= 6;

const uint8	kSyncBitMask = 0x1 << kSyncBit;
const uint8 kProximityBitMask = 0x1 << kProximityBit;
const uint8 kPointerBitMask = 0x1 << kPointerBit;
const uint8 kButtonBitMask = 0x1 << kButtonBit;
const uint8 kPressureSignBitMask = 0x1 << kPressureSignBit;

status_t
convertinfotostruct(const uint32 *data, tablet_info &info)
{	
}

status_t
convertpositiontostruct(const uint8 *data, tablet_position &info)
{	
	if (!(data[0] & kSyncBitMask))
	{
		//printf("convertpositiontostruct - early return, no sync bit\n");
		return B_ERROR;
	}
	
	info.proximity = (data[0] & kProximityBitMask) >> kProximityBit;
	info.stylus = (data[0] & kPointerBitMask) >> kPointerBit;
	info.buttonflag = (data[0] & kButtonBitMask) >> kButtonBit;

	info.x = (data[0] & 0x3) << 14;
	info.x += (data[1] & 0x7f) << 7;
	info.x += (data[2] & 0x7f);
	
	info.buttons = (data[3] & 0x78) >> 3;
	
	info.y = (data[3] & 0x3) << 14;
	info.y += (data[4] & 0x7f) << 7;
	info.y += (data[5] & 0x7f);
		
	info.pressuresign = (data[6] & kPressureSignBitMask) >> kPressureSignBit;
	info.pressuredata = (data[6] & 0x3f);
}

//
// Retrieve a line of data from the tablet.  It should be
// terminated with a <cr>
void
GetLine(serial_port &portid, char *buff, const long buffLen)
{
	//return;
	
	bool done = false;
	char *ptr = buff;
	uint8 aChar;
	long bytestoread=0;
	long totalread = 0;
	
	while(!done && (totalread < buffLen))
	{
		bytestoread = SP_WaitForInput(portid);
		//printf("BytesToRead - %d\n", bytestoread);
		while(bytestoread)
		{
			long numRead = SP_Read(portid, &aChar, 1);
			if (numRead>0)
			{
				//printf("%c",aChar);
				//fflush(stdout);
				if (aChar == '\r')
				{
					*ptr = '\0';
					done = true;
					break;
				} else
					*ptr = aChar;
				ptr++;
				totalread ++;
			}	
			bytestoread -= numRead;
		}
	}	
}

status_t
GetBytes(serial_port &portid, uint8 *bytes, const uint32 numBytes)
{
	long bytestoread = numBytes;
	long numRead;
	uint8 *buffPtr = bytes;
	
	long byteswaiting = SP_WaitForInput(portid);
	
	if (byteswaiting < 1)
	{
		return B_ERROR;
	}

	while (bytestoread)
	{
		numRead = SP_Read(portid, buffPtr, bytestoread);
		buffPtr += numRead;
		bytestoread  -= numRead;
	}

	return B_NO_ERROR;
}


int	
SendCommand(wacom_tablet &tablet, const char *cmd)
{
	return SP_Write(*tablet.sport, cmd, strlen(cmd));
}


status_t
GetModel(wacom_tablet &tablet, char *buff)
{
	buff[0] = '\0';
	
	SendCommand(tablet, "~#\r");
	GetLine(*tablet.sport, buff,32);

	return B_NO_ERROR;
}

status_t	
GetMaxCoordinates(wacom_tablet &tablet, int32 &x,int32 &y)
{
	char buff[32];
	
	SendCommand(tablet, "~C\r");
	GetLine(*tablet.sport, buff,32);
	//printf("Max Coordinates: %s\n", buff);
	
	return B_NO_ERROR;
}

status_t
GetCurrentSettings(wacom_tablet &tablet, char *buff, const int32 buffLen)
{
	SendCommand(tablet, "~R\r");
	GetLine(*tablet.sport, buff,buffLen);
	
	return B_NO_ERROR;
}

status_t	
GetTabletInfo(wacom_tablet &tablet, tablet_info &info)
{
	// Send the individual request command
	SendCommand(tablet, "RQ1\r");
	
	// Try to get the position information
	uint32 buff;
	status_t status = GetBytes(*tablet.sport, (uint8 *)&buff, 4);
	
	if (status == B_ERROR)
	{
		//printf("No bytes read.\n");
		return status;
	}

	status = convertinfotostruct(&buff, info);
	
	if (status == B_ERROR)
	{
		//printf("Invalid Data.\n");
		return status;
	}
		
	return B_NO_ERROR;
}

status_t	
GetLocation(wacom_tablet &tablet, int &x, int &y, int &buttons)
{	
	tablet_position info;
	GetPosition(tablet, info);
	
	x = info.x;
	y = info.y;
	buttons = info.buttons;
	
	return B_NO_ERROR;
}

status_t	
GetPosition(wacom_tablet &tablet, tablet_position &position)
{	
	uint8 buff[128];

	SendCommand(tablet, "RQ1\r");
	status_t status = GetBytes(*tablet.sport, buff, 7);
	position.status = status;
		
	if (status == B_ERROR)
	{
		//printf("No bytes read.\n");
		return status;
	}

	convertpositiontostruct(buff, position);
	
	return B_NO_ERROR;
}

status_t
SetASCIIFormat(wacom_tablet &tablet)
{
	SendCommand(tablet, "AS0\r");
}

status_t
SetBinaryFormat(wacom_tablet &tablet)
{
	SendCommand(tablet, "AS0\r");
	return B_NO_ERROR;
}

status_t
SetPointMode(wacom_tablet &tablet)
{
	SendCommand(tablet, "PO\r");
	return B_NO_ERROR;
}

status_t
SetRelativeMode(wacom_tablet &tablet)
{
	SendCommand(tablet, "DE1\r");
	return B_NO_ERROR;
}

status_t
SetAbsoluteMode(wacom_tablet &tablet)
{
	SendCommand(tablet, "DE0\r");
	return B_NO_ERROR;
}

status_t
SelectIISCommandSet(wacom_tablet &tablet)
{
	SendCommand(tablet, "$\r");
	return B_NO_ERROR;
}

status_t
SelectIVCommandSet(wacom_tablet &tablet)
{
	SendCommand(tablet, "#\r");
	return B_NO_ERROR;
}

status_t
Select1201CommandSet(wacom_tablet &tablet)
{
	SendCommand(tablet, "@\r");
	return B_NO_ERROR;
}

status_t
SetAlwaysTransmit(wacom_tablet &tablet, const int8)
{
	SendCommand(tablet, "AL%d\r");
	return B_NO_ERROR;
}

status_t	
SetTransmissionInterval(wacom_tablet &tablet, const int16)
{
	return B_NO_ERROR;
}


status_t
Reset(wacom_tablet &tablet)
{
	SendCommand(tablet, "RE\r");

	return B_NO_ERROR;
}
