/* --------------------------------------------------------------------
/	File:	wacom.c
/	$Revision: 1.63 $
/	Author: William Adams
/	Date:  Jun 16, 1997
/	Copyright (c) 1994 by Be Incorporated.  All Rights Reserved.
/------------------------------------------------------------------- */

#include <KernelExport.h>
#include <SupportDefs.h>
#include <Drivers.h>
#include <termios.h>
#include <fcntl.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <malloc.h>

#include "wacom_driver.h"
#include "tablet.h"
#include "serial.h"

#define ddprintf dprintf

static long wacom_open(const char *name, uint32 flags, void **cookie);
static long wacom_close(void *cookie);
static long wacom_free(void *cookie);
static long wacom_control(void *cookie, uint32 msg, void *buf, size_t len);
static long wacom_read(void *cookie, off_t pos, void *buf, size_t *count);
static long wacom_write(void *cookie, off_t pos, const void *buf, size_t *count);

static wacom_tablet *	CreateWacomTablet(const char *name, wacom_tablet **);
static void	DestroyWacomTablet(wacom_tablet *tablet);

wacom_tablet	*gtablet=0;

/* --------------------------------------------------------------------
/	initialize the serial tablet
/ */

/*
	Function: CreateWacomTablet
	
	This function creates a internal representation of the graphics
	tablet.  There is only one instance for this driver.  That means 
	this driver can only deal with a single tablet.
	
	The object is reference counted, so you can open the driver as many 
	times as you like, from any application.  As each application closes
	its connection to the tablet, the reference count is decremented.
	
*/

wacom_tablet * CreateWacomTablet(const char *name, wacom_tablet **tablet)
{
	if (0 == *tablet)
	{
		*tablet = (wacom_tablet *)malloc(sizeof(wacom_tablet));
		gtablet = *tablet;
		CreateSerialPort(&gtablet->sport);
		SP_Open(*gtablet->sport, "serial2");
		SP_SetBlocking(*gtablet->sport, false);
		SP_SetTimeout(*gtablet->sport, 250000);		// Timeout of a quarter second
		gtablet->numClients = 1;
	} else	// Increment the reference count
	{
		gtablet->numClients++;
	}
	
	return *tablet;
}

/*
	Function: DestroyWacomTablet
	
	The gtablet is a reference counted thing.  Whenever
	you destroy it, you first decrement the count.  If the 
	count goes to 0, that means there are no more references
	to the structure, and it should be deleted.  Otherwise,
	let the structure stay in place, because there are still
	references.
*/

void
DestroyWacomTablet(wacom_tablet *tablet)
{
	if (!tablet)
		return;
		
	if (tablet->numClients >= 1)
		tablet->numClients--;
	
	if (tablet->numClients == 0)
	{
		DestroySerialPort(tablet->sport);
		free(tablet);
		tablet = 0;
	}
}

static void read_wacom_settings()
{
	int		mouse_prefs;
	int		loop;

	if ((mouse_prefs = open("/boot/system/settings/wacom_settings", 0)) >= 0) 
	{
/*
		read(mouse_prefs, &mouse_type, sizeof(int));
		if ((mouse_type < MOUSE_1_BUTTON) || (mouse_type > MOUSE_3_BUTTON))
			mouse_type = MOUSE_3_BUTTON;
		read(mouse_prefs, &mouse_mapping, sizeof(mouse_map));
		for (loop = 0; loop < 3; loop++) {
			if ((mouse_mapping[loop] != 1) && (mouse_mapping[loop] != 2) &&
				(mouse_mapping[loop] != 4))
				mouse_mapping[loop] = 1 << loop;
		}
		read(mouse_prefs, &accel, sizeof(uchar));
		read(mouse_prefs, &accel_factor, sizeof(uchar));
		read(mouse_prefs, &speed, sizeof(uchar));
		read(mouse_prefs, &click_speed, sizeof(bigtime_t));
		if (click_speed < 100000)
			click_speed = 500000;
*/
		close(mouse_prefs);
	}
}

static void write_wacom_settings()
{
	int			mouse_prefs;

	if ((mouse_prefs = open("/boot/system/settings/wacom_settings", O_CREAT | O_TRUNC | O_RDWR, 0777)) >= 0) 
	{
#if 0
		write(mouse_prefs, &mouse_type, sizeof(int));
		write(mouse_prefs, &mouse_mapping, sizeof(mouse_map));
		write(mouse_prefs, &accel, sizeof(uchar));
		write(mouse_prefs, &accel_factor, sizeof(uchar));
		write(mouse_prefs, &speed, sizeof(uchar));
		write(mouse_prefs, &click_speed, sizeof(bigtime_t));
#endif
		close(mouse_prefs);
	}
}


/* 
	init_hardware - one-time hardware initialization
*/
long
init_hardware (void)
{
	return B_NO_ERROR;
}


/* 
	Function: init_driver

	This function is first called when the driver is loaded, and before
	any of the other functions are used.  You would put in anything special
	to initialize the driver.  You could perhaps put in code to setup the
	global tablet structure, but there's really no point until the open call
	is used.
*/
long
init_driver (void)
{
	return B_NO_ERROR;
}

/*
	Function: wacom_open
	
	When in user code you use the open() function, this routine ends
	up being called.
	
	The name parameter indicates by what name the driver was called.
	the flags field indicates what read/write permissions were used.
	You are supposed to create and return the cookie argument.
	
	This cookie will be handed back to you in the read/write/control
	functions.  You generally create a structure which is appropriate for 
	the driver to maintain state information.
	
	In this particular driver the structure that is created is a global
	on. This driver can be opened as many times as the clients like without
	restriction.  But there is only one tablet that it can serve, so all
	the opens will result in controlling the same tablet.
*/
static long
wacom_open(const char *name, uint32 flags, void **cookie)
{
	/* read_tablet_settings(); */
	wacom_tablet *tablet;
	CreateWacomTablet("name", &tablet);
	*cookie = tablet;
	
	return B_NO_ERROR;

err:
	ddprintf ("wacom initialization error\n");
	wacom_close(NULL);
	
	return B_ERROR;
}

/* 
	Function: wacom_close
	
	Whenever a client calls close(fd), this function ends up being called.
	There really isn't much to do.  We free the cookie in the wacom_free
	routine, so we don't do anything special here.
*/

static long
wacom_close(void *cookie)
{
	return B_NO_ERROR;
}

/*
	Function: wacom_free
	
	This function is called after the wacom_free function by the kernel.
	It is intended to allow you to free up the cookie if you had created
	it originally.  Here we call the DestroyWacomTablet() function.
	That function will decrement a reference count, and free the memory
	if necessary.
*/

static long
wacom_free(void *cookie)
{
	wacom_tablet	*tablet = (wacom_tablet *)cookie;
	DestroyWacomTablet(tablet);
	
	return 0;
}


/*
	Function:	wacom_control

	This is the function that is ultimately called when the
	client does a ioctl() call in their code.  The intention
	is to control the device.
	
	We use it to load and save the parameters file, and to get
	tablet positional information, as well as getting and setting
	various parameters.
	
	If you wanted to get more specific inforamtion out of the tablet,
	or if there are more features that you want to expose to client
	code, then this is the place to go.
*/

static long
wacom_control(void *cookie, uint32 msg, void *buf, size_t len)
{
	tablet_info *info = (tablet_info *)buf;
	tablet_position *position = (tablet_position *)buf;
	wacom_tablet	*tablet = (wacom_tablet *)cookie;
	status_t status;
	
	switch (msg) 
	{			
		case TABLET_GETINFO:
			status = GetTabletInfo(*tablet, *info);
		break;
		
		case TABLET_SETINFO:
		break;
		
		case TABLET_GETPOSITION:
			status = GetPosition(*tablet, *position);
		break;
		
		case TABLET_SAVESETTINGS:
		break;
		
		case TABLET_READSETTINGS:
		break;
		
		default:
			return B_ERROR;
	}

	return B_NO_ERROR;
}


/* 
	Function: wacom_read
	
	We don't do anything special when it comes to reading from the 
	tablet.  It is not a character device, so there's really nothing 
	to read.
	
	It might be interesting to return some sort of string that contains
	information about the driver.  Possibly a version string as well as
	the current tablet information, which port it's connected to, and 
	possibly the current position.
*/

static long
wacom_read(void *cookie, off_t pos, void *buf, size_t *count)
{	
	return EINVAL;
}


/*
	Function: wacom_write
	
	The tablet doesn't really have anything to write to through
	this call.  You might possibly think of allowing for a text based
	interface that could be used to write various commands to the tablet
	to set parameters.  Or quite possibly you would allow the client code
	to write directly to the tablet using this interface.
	
*/

static long
wacom_write(void *cookie, off_t pos, const void *buf, size_t *count)
{
	return EINVAL;
}




/*
	driver-related structures
*/

/*
	Every device driver must create one of these structures.  You must fill it
	in with the various routines that will be used to do the actual work.
*/

static device_hooks wacom_device = {
	wacom_open,		/* -> open entry point */
	wacom_close,	/* -> close entry point */
	wacom_free,		/* -> free cookie entry point */
	wacom_control,	/* -> control entry point */
	wacom_read,		/* -> read entry point */
	wacom_write		/* -> write entry point */
};

/* 
	This is a simple way to publish the name of the driver.
	If you had multiple names that this driver could deal with,
	then this structure would be modified.
*/

static const char	*wacom_name[] = { "wacom", NULL };

/*
	The simplest case of this routine is to simply publish the static
	structure that contains the name of the driver we allow to be opened.
	
	If you wanted to deal with more than one tablet at a time, then this 
	routine should create the structure on the fly and publish a structure
	that contains a name for each of the tablets.
*/

const char **
publish_devices()
{
	return wacom_name;
}

/*
	Function: find_device
	
	This funtion is called by the kernel.  It uses the returned structure
	to make various calls to the device.  The function pointers must be filled
	in with the pointers to the appropriate routines before your driver will
	work correctly.

	It is easiest to simply setup a static structure with the pointers and return
	a pointer to that structure from here.
*/

device_hooks *
find_device(const char *name)
{
	return &wacom_device;
}
